from numpy import pi, sqrt, float64
#[15.08.09] Follow new propoposed constants from IAU resoluiton


gN = 6.67408e-08            #NIST, 2014 CODATA
amu = 1.660539040e-24       #NIST, 2014 CODATA
kB = 1.38064852e-16         #NIST, 2014 CODATA
sB = 5.670367e-5            #NIST, 2014 CODATA

#astronomical constants
au = 1.49597870700e13       #IAU 2009, resolution B2
pc = 3.0857e18

rSun = 6.957e10             #IAU 6.96342e10
gmSun = 1.3271244e26        #IAU
gmEarth = 3.986004e20   #2015 IAU
gmJup =   1.2668653e23  #2015 IAU

mSun = gmSun /gN
mEarth = gmEarth /gN
mJup = gmJup /gN

yr = 2*pi/sqrt(gmSun/au**3)
day = 86400e0 #seconds


def units_conv (sQ, unit_in='cgs', unit_out='cgs'):

    i = len(unit_out)
    while i>0:
        if unit_out[i-1].isalpha(): i -= 1
        else: break

    unout = unit_out[i:]

    if i!=0:
        pre = float64(unit_out[:i])
    else:
        pre = 1.

    Qp = {}
    if sQ=='pressure':
        Qp['cgs'] = 1.
        Qp['dyn/cm2'] = 1.
        Qp['mbar'] = 1e-3
        Qp['bar'] = 1e-6
        Qp['Pa'] = 0.1
        Qp['SI'] = 0.1
    elif sQ=='density':
        Qp['cgs'] = 1.
        Qp['g/cm3'] = 1.
        Qp['SI'] = 1e3
        Qp['kg/m3'] = 1e3
    elif sQ=='length' or sQ=='height':
        Qp['cgs'] = 1.
        Qp['cm'] = 1.
        Qp['km'] = 1e-5
        Qp['m'] = 1e-2
        Qp['SI'] = 1e-2

    return Qp[unout]/Qp[unit_in] /pre
    

