from numpy import *
import sys
sys.path.append('./../Genfuns/')
import genfuns3 as gf
import utilities as util
import cgs
import scipy.interpolate as sciip
import scipy.optimize as sciop

def get_profile (fname, returnData=False,
        dunits={'pressure':'bar', 'height':'cm', 'density':'cgs'}, Prange='all',order='cubic'):
    """
    [17.01.26]  :I re-arranged this into log units for pressure
    """
    scol, scolunit = util.readhdr_myTable (fname)
    data = loadtxt(fname) 

    #given in mbar!
    iX = scol.index('p')
    pres = data[:,iX] \
            *cgs.units_conv('pressure',scolunit[iX],dunits['pressure'])

    if Prange=='all':
        ii = ones((len(pres)),dtype=bool)
    else:
        ii = (pres<Prange[0]) * (pres>Prange[1])
        #also include the neighbors
        ii = roll(ii,2) +roll(ii,-2)

    lpdata = log(pres[ii])

    #data given in km
    iX = scol.index('z')
    zdata = data[ii,iX] \
            *cgs.units_conv('length',scolunit[iX],dunits['height'])

    iX = scol.index('T')
    Tdata = data[ii,iX]

    #data given in SI
    iX = scol.index('rho')
    rhodata = data[ii,iX] \
            *cgs.units_conv('density',scolunit[iX],dunits['density'])


    spline_pz = sciip.interp1d(zdata[::-1], lpdata[::-1],  
                kind=order, bounds_error=False)

    spline_zp = sciip.interp1d(lpdata, zdata,  
                kind=order, bounds_error=False)


    spline_Tz = sciip.interp1d(zdata[::-1], Tdata[::-1],  
                kind=order, bounds_error=False)

    spline_rhoz = sciip.interp1d(zdata[::-1], rhodata[::-1],  
                kind=order, bounds_error=False)
    print('[planprofile]:spline interpolation done')

    if returnData:
        return [spline_pz, spline_zp, spline_Tz, spline_rhoz], [zdata, lpdata, Tdata, rhodata]
    else:
        return spline_pz, spline_zp, spline_Tz, spline_rhoz


def delrho (lp, Xv, spline_zp, spline_rhoz, spline_Tz, rho_eq):
    z = spline_zp(lp)
    rhoX = spline_rhoz(z) *Xv
    temp = spline_Tz(z)
    rhoEq = rho_eq(temp)

    return rhoX/rhoEq -1



def plot_profile (fname, Prange, species, mvap, Xn=0, plotlmfp=False, doPlot=True):
    import pylab as pl 

    [spline_pz, spline_zp, spline_Tz, spline_rhoz], [zdata, pdata, Tdata, rhodata] =\
        get_profile(fname,returnData=True,Prange=Prange)


    lparr = linspace(log(Prange[0]),log(Prange[1]),100)
    parr = exp(lparr)

    zarr = spline_zp (lparr)
    rhoarr = spline_rhoz (zarr)
    Tarr = spline_Tz (zarr)

    Peq = gf.es_X(Tarr, species)
    rhoeq =  gf.rhoEq_X(Tarr, species, mvap)

    def rho_eq (T):
        return gf.rhoEq_X(T, species, mvap)

    if doPlot:
        fg = pl.figure(figsize=(5,4))
        fg.subplots_adjust(top=0.9)
        ax = fg.add_subplot(111)
        ax2 = ax.twiny()

        ax.semilogy(Tdata, exp(pdata), '.', c=(1,.5,.5))
        ax2.loglog(rhodata, exp(pdata), '.', c='0.5')

        if plotlmfp:
            lmfp = gf.l_mfp(parr*1e6, Tarr)
            ax2.plot(lmfp, parr, '-', c='0.5')

        l1 = ax.semilogy(Tarr,   parr, 'r-', lw=.5)
#       l1a = ax.semilogy(Tarr,  gf.es_X(Tarr, species), 'r--', lw=.5)
        l2 = ax2.semilogy(rhoarr,parr, 'k-', lw=.5)

        if species!='':
            #ax.semilogy(Tarr, Peq, 'r--')
            l3 = ax2.semilogy(rhoeq, parr, 'b--')
            l4 = ax2.semilogy(rhoarr*Xn, parr, 'b-')
            ax.legend( (l1[0],l2[0],l3[0],l4[0]), ['T','rho','rhoEq('+species+')','rho('+species+')'], 
                    loc=3)
        else:
            ax.legend( (l1[0],l2[0]), ['T', 'rho'], loc=3)

        sname = fname[:fname.index('/')]
        ax2.text(0.05, 0.95, sname, transform=ax2.transAxes, fontsize=14, 
                va='top', ha='left', bbox=dict(fc='w', lw=0, alpha=0.8))


        ax.set_ylim(Prange)
        ax.set_ylabel('pressure [bar]')
        ax.set_xlabel('Temperature [K]')
        ax2.set_xlabel('density [cgs]')


    if species!='' and Xn!=0:
        lpbot = sciop.brentq(delrho, log(Prange[1]), log(Prange[0]), 
                                args=(Xn, spline_zp, spline_rhoz, spline_Tz, rho_eq))
        pbot = exp(lpbot)
        if doPlot:
            ax.axhline(pbot, ls='--')
            print('bottom atmosphere at', pbot)

    if doPlot:
        gname = 'profile.pdf'
        fg.savefig(gname)
        pl.close()

    if species!='' and Xn!=0:
        return pbot
