from numpy import *
import pylab as pl 
import sys
sys.path.append('../Genfuns/')
import cgs
import scipy.integrate as sciint
from matplotlib.ticker import MultipleLocator
import subprocess as sp

def T4_tau (tau, Tint=100., g1=0.158, Tstar=5040., f=0.25,
            Rstar = 0.756*cgs.rSun, rp=0.1*cgs.au, **kwargs):
    """
    Guillot (2010), equation 1, 29
    """
    g3 = g1*sqrt(3.)
    Tirr = sqrt(Rstar/rp) *Tstar

    T4 = 0.75*Tint**4    *(2./3 +tau) +\
         0.75*Tirr**4 *f *(2./3 +1/g3 +(g3/3. -1/g3)*exp(-g3*tau))
    return T4


def eos_rho (P, T, **ppars):
    mu = ppars['mu']
    return P/(cgs.kB*T/(mu*cgs.amu))


def get_T (tau):
    T4 = T4_tau(tau, **ppars)
    return T4**0.25

def rho_tau (tau, ppars):
    gz = ppars['gz']
    kapIR = ppars['kapIR']
    Pgas = tau *gz/kapIR
    Tgas = get_T(tau)
    rhog = eos_rho(Pgas, Tgas, **ppars)
    return rhog


def get_tau (pres, ppars):
    tau = ppars['kapIR'] *pres /ppars['gz']
    return tau

def dz_dP (z, p, ppars):
    tau = get_tau(p, ppars)
    rho = rho_tau(tau,ppars)
    dzdP = 1/(rho*ppars['gz'])
    return dzdP


#parameters (following Line) #ppars = {'Tint':100,'mu':2.34,'kapIR':0.03,'gz':2192}

#parameters for HJ following Michiel
ppars = {'Tint':500,'mu':2.34,'kapIR':3.00,'gz':2192,'Rstar':cgs.rSun,
        'rp':0.05*cgs.au,'Tstar':5777., 'name':'HotJupiter'}

#parameters for GJ1214b
ppars ={'Tint':60, 'kapIR':0.03,'rp':0.0143*cgs.au,'g1':0.038,'g2':0.038,
        'Tstar':3026.,'name':'GJ1214b','Rstar':0.2064*cgs.rSun}

#same as above HJ2 model
ppars = {'name':'HJ2','Tint':500,'mu':2.34,'kapIR':0.3,'gz':2192,'Rstar':cgs.rSun,
        'rp':0.05*cgs.au,'Tstar':5777.}


#add parameter from cpars
ddir = ppars['name']+'/'
sys.path.append(ddir)
import cloudpars as cpars
ppars['gz'] = cpars.gz
ppars['mu'] = cpars.mu

Npt = 500
Parr = 10**(6+linspace(log10(cpars.Prange[1]), log10(cpars.Prange[0]), Npt))

tauarr = get_tau(Parr, ppars)
Tarr = get_T (tauarr)

rhoarr = eos_rho (Parr, Tarr, **ppars)

z0 = 0.
zarr = sciint.odeint(dz_dP, z0, Parr, args=(ppars,))

#take zarr relative to tau=2/3
i0=abs(tauarr-2./3).argmin()
zarr = zarr[i0][0] - zarr[:,0]

fname = 'profile'+ppars['name']+'.txt'
f = open(fname,'w')
f.write('#cols::[z,tau,rho,T,p]\n')
f.write('#colunits::[cm,,g/cm3,K,bar]\n')
for i in range(Npt):
    sft = '{:17.10e}'+2*' {:12.5e}'+' {:11.5f}'+' {:12.5e}'
    f.write(sft.format(zarr[i],tauarr[i],rhoarr[i],Tarr[i],Parr[i]/1e6))
    f.write('\n')
f.close()

#Now we copy the profile to the directory
sp.call(['mv', '-v', fname, ddir+'profile.txt'])

#Now we're plotting the profile
import planprofile as planprof

#if this gives a brentq error; put a "0" before cpars.Xvbot
planprof.plot_profile (ddir+'profile.txt', cpars.Prange, cpars.species, cpars.mvap,
                        cpars.Xvbot, doPlot=True)

