from numpy import *
import sys
sys.path.append('./../Profiles/')
sys.path.append('./../Genfuns/')
import genfuns3 as gf
import utilities as util
import cgs
import cloudcore
import config as cfg
import copy
import cloudplot

###
# Program starts here
###

if len(sys.argv)>1:
    sdescr, planetName = sys.argv[1].split(':')
    if sdescr!='PlanetName':
        print('[cloud-model]:No valid planet')
        sys.exit()
else:
    planetName = 'GJ1214b'# 'HJ2'# 'HotJupiter'#

ddir = './../Profiles/'+planetName+'/'
sys.path.append(ddir)
import cloudpars as cpars

if len(sys.argv)>1:

    doPlot = False
    cparsnew = util.make_dict_entries(sys.argv[1:])

    for key in cparsnew:
        setattr(cpars, key, cparsnew[key])
        print(cpars, key, cparsnew[key])

else:
    doPlot = True
    cpars.pextent = 30. #K8



#initialize!
xg, Yg = cloudcore.initialize (cpars.pextent, ddir, doPlot)

irunP = 0
fnucL = [1e0, 1e0]
ffacL = [1e-12, 1e0]
ttolL = [cfg.tol1, 1e-4]
irun = 0; tdel = 1.
iphase = 0; itol = 0
delxB = cfg.delxB0
delxT = cfg.delxT0
renormalize = False
while True:

    #iphase changes
    if iphase==0 and irun>=1 and bvpsol.success and fnuc>=1.0:
        irun = 0; itol = 0
        iphase = 1
        print('[cloud-model]:Entering phase 1...')

    elif iphase==1 and irun>=len(ffacL):
        if doPlot: out = cloudplot.plot_result(bvpsolG, cpars, xarr=bvpsolG.x)
        print('[cloud-model]:entering phase2!')
        cfg.nodesX *= 2
        itol = 10
        irun = 0
        iphase = 2

    elif iphase==2 and (irun>=len(ttolL) or tdel<1e-2 and acceptSol):
        if doPlot: out = cloudplot.plot_result(bvpsolG, cpars, xarr=bvpsolG.x)
        iphase = 3
        print("[cloud-model]:Entering phase 3...")
        xmin = 0.#this is justed for the top!!
        cfg.nodesX *= 2


    if iphase==1:
        ttol = ttolL[0]
        cfg.ffac = ffacL[irun]
    elif iphase==2:
        ttol = ttolL[irun]
    elif iphase==0:
        ttol = ttolL[0]*2**itol
        cfg.ffac = 0.
        fnuc = fnucL[irun]

    if renormalize==False:
        #remove negative entries from Yn
        Yn = cloudcore.prepare_Ynew (Yg)

        #adjust the grid
        if iphase==3:
            if delxT>cfg.delxTm and Yg[3][-1]/Yg[3].max()>1e-10:# and xg.max()<xmax:
                if acceptSol==False:
                    delxT /= 2.
                else:
                    delxT *= 1.08
                #adjust the top, extending with delxT
                #but ensure not to overshoot xmax 
                if xg.max()+delxT>cfg.xmax: delxT = cfg.xmax-xg.max()
                xn = xg.min() +(xg-xg.min())*(xg.max()-xg.min()+delxT)/(xg.max()-xg.min())

            elif delxB<cfg.delxBm:
                if acceptSol==False: delxB /= 2.0

                #adjust the lower range, while keeping the shape
                xn = xg.max() +(xg-xg.max())*(xg.max()-xg.min()-delxB)/(xg.max()-xg.min())
            else:
                print('[cloud-model]:finished!!')
                break

        #iphase<>3
        else:
            xn = copy.copy(xg)



    #code works best when xnodes gently increases
    #something like this...
    xnodes = min(cfg.nodesX, sqrt((1+itol)*len(xg) *cfg.nodesX))


    #here the hard work is done
    if renormalize:
        bvpsol = cloudcore.iterate_solution(xn, Yn, ttol, 0.5*(1+iphase)*xnodes)
    else:
        bvpsol = cloudcore.iterate_solution(xn, Yn, ttol, xnodes)


    #will fail when numbers become too negative
    con3 = cloudcore.bvp_negative (bvpsol, iphase)
    acceptSol = False

    if (bvpsol.success==False or con3==False):
        #con1==False or con2==False or
        if iphase==0:
            itol += 1
            if irun==0:
                fnucL[irun] /= 1e1
                if fnucL[irun]<1e-40:
                    print('fnucL<1e-40')
                    import ipdb; ipdb.set_trace()
            else:
                fnuc_new = sqrt(fnucL[irun-1]*fnucL[irun])
                ndel = (fnuc_new-fnuc)/fnuc
                if ndel>-1e-4:
                    print('cannot get initial profile in phase0')
                    import ipdb; ipdb.set_trace()

                fnucL.insert(irun, fnuc_new)

        elif iphase==1:
            if irun==0:
                print("lower ffac [ffac,xn-top]", cfg.ffac, xn[-1])
                xn = xn *(0.8)**itol
                itol += 1
                ffacL[irun] /= 1e2
                if ffacL[irun]<1e-40:
                    print('ffacL<1e-40')
                    import ipdb; ipdb.set_trace()
            else:
                ffac_new = sqrt(ffacL[irun-1]*ffacL[irun])
                fdel = (ffac_new-cfg.ffac) /cfg.ffac
                if fdel>-1e-5:# and any(bvpsol.y[1][-10:]<0):
                    print('[cloud-model]ERROR:too close!!')
                    print('fdel = ', fdel)
                    import pdb; pdb.set_trace()
                else:
                    ffacL.insert(irun, ffac_new)

        elif iphase==2:
            itol += 1
            if irun==0:
                if itol>5:
                    print("cannot achieve solution in phase 2")
                    import pdb; pdb.set_trace()
            elif tdel<1e-5:
                print('[cloud-model]:too close or too many particles! Accepting solution')
                acceptSol = True#the best we can do?!
            else:
                ttol_new = sqrt(ttolL[irun-1]*ttolL[irun])
                tdel = (ttol_new-ttol)/ttol
                ttolL.insert(irun, ttol_new)

    #check w/r normalization needed
    #if so run again (acceptSol=False)
    else:
        renormalize = cloudcore.normalize_norms (Yn, bvpsol, iphase)
        if renormalize==False: acceptSol = True

    #adjust the grid in phase1 already when condensate density becomes negative
    if iphase==1 and acceptSol and bvpsol.y[3][-1]<0:
        acceptSol = False
        renormalize = True
        xn *= 0.99

    #all signs are green
    if acceptSol:
        #copy solution; assign new guess
        bvpsolG = copy.deepcopy(bvpsol)
        xg = bvpsol.x #g/G means "good" solution
        Yg = bvpsol.y

        #move on
        irun += 1
        itol = 0

        #plot results (time consuming)
        if doPlot:
            if (irun>irunP+1+5*iphase) or (iphase==1 and cfg.ffac==1.) or (iphase==2 and ttol==ttolL[-1]):
                out = cloudplot.plot_result(bvpsolG, cpars, xarr=bvpsolG.x)
                irunP = irun

    #print some results to screen
    if acceptSol:
        spre = '\033[1m'
        sapp = '\033[0m'
    else:
        spre = ''; sapp = ''

    fmt = spre+'%2d %4d %5d %1d %1d %1d %9.2e %9.2e %12.4e %12.4e'+sapp
    if iphase==0:
        print(fmt%(iphase, irun, len(xg), acceptSol, bvpsol.status, con3, fnuc, ttol, 0, 0))
    elif iphase==3:
        print(fmt%(iphase, irun, len(xg), acceptSol, bvpsol.status, con3, bvpsol.y[3].min()/bvpsol.y[3].max(), ttol, xn.min(), xn.max()))
    else:
        print(fmt%(iphase, irun, len(xg), acceptSol, bvpsol.status, con3, bvpsol.y[3].min()/bvpsol.y[3].max(), ttol, cfg.ffac, xn.max()))


#if doPlot:
out = cloudplot.plot_result(bvpsolG, cpars, xarr=bvpsolG.x)
cparL = ['aseed','mseed','mvap','xvbot','Kzz','rhoint','gz','past','sigast','sigdotN','mPlanet']

print("[cloud-model]:Writing final")
data, shdr = cloudplot.plot_result(bvpsolG, cpars, cparL=cparL, fmt='pdf', doWrite=True, Npt=5000, finalPlot=True)
