import pylab as pl
import sys
sys.path.append('./../Profiles/')
sys.path.append('./../Genfuns/')
import cloudcore
import genfuns3 as gf
import numpy as npy
import cgs
import subprocess as sp

def plot_result (bvpsol, cpars, cparL=[], fmt='png', doWrite=False, Npt=100, xarr='',
                    finalPlot=False):

    if type(xarr)==str:
        xarr1 = npy.linspace(bvpsol.x.min(), bvpsol.x.max(), Npt)
        xarr2 = npy.linspace(xarr1[-2],xarr1[-1],Npt/10)
        xarr3 = npy.linspace(xarr2[-2],xarr2[-1],Npt/10)
        xarr = npy.concatenate((xarr1[:-1], xarr2[1:-1], xarr3[1:]))
        Npt = len(xarr)

    zarr = cloudcore.x_to_z(xarr)

    Mc,xc,xv,Mn,xn,rhog,Tarr,xeq,parr = cloudcore.get_from_bvpsol (xarr, bvpsol)
    ssat = xv/xeq
    vth = gf.v_th(Tarr, cpars.mvap)
    diffM = gf.diff_i(parr, Tarr)

    rhoNdot = gf.nnucl_spont (rhog, parr, cpars.mseed, cpars.sigdotN,
                                cpars.past, cpars.sigast, cpars.gz)

    #particle's physical properties
    np, mp, ap, tstop, vsettl =\
            gf.get_pprop (xn,xc,Tarr,rhog,cpars.mseed,cpars.rhoint,cpars.gz,
                            mgas=cpars.mgas, EpsteinApprox=cpars.EpsteinApprox)


    kappa_geo = npy.pi *np *ap**2
    dtau = kappa_geo[1:] *npy.diff(zarr)
    tau = npy.cumsum(dtau[::-1])[::-1]
    tau = npy.concatenate((tau,[0]))

    if finalPlot and cpars.EpsteinApprox==False:
        tEp = ap*cpars.rhoint/(gf.v_th(Tarr,cpars.mu*cgs.amu)*rhog)
        if all(abs(tEp/tstop -1)<1e-8):
            print("All Epstein!")
        else:
            print((tEp/tstop).min(), (tEp/tstop).max())

    if finalPlot:
        #calculate tau according to the limb
        Nz = len(zarr)
        tautrn = npy.zeros((Nz))
        Rplanet = npy.sqrt(cpars.mPlanet*cgs.gN/cpars.gz)
        for k in range(Nz):
            tautrn[k] = gf.calc_tau_limb(zarr[k:], kappa_geo[k:], Rplanet)

        data = npy.array([zarr,parr,Tarr,rhog,Mc,xc,xv,Mn,xn,xeq,1-xeq/xv,ap,np,rhoNdot,tau,tautrn]).T
        shdrL = ['z','p','T','rhog','Mc','xc','xv','Mn','xn','xeq','ss','ap','np','rhoNdot','tau','tautrn']
        i8 = abs(xc-1e-8).argmin()
        print('[testcloud8]min,p8,max:', parr.min(), parr[i8], parr.max())
    else:
        data = npy.array([zarr,parr,Tarr,rhog,Mc,xc,xv,Mn,xn,xeq,1-xeq/xv,ap,np,rhoNdot,tau]).T
        shdrL = ['z','p','T','rhog','Mc','xc','xv','Mn','xn','xeq','ss','ap','np','rhoNdot','tau']


    #mass flux conservation!
    Mv = -Mc

    if doWrite:
        g = open('output.dat', 'w')
        line = '#cols::['+','.join(shdrL)+']'
        g.write(line+'\n')
        line = '#pars::[planet,species]:: '
        line += '{} {}'.format(cpars.name, cpars.species)
        g.write(line+'\n')
        line = '#pars::[doCoag]:: '
        line += '{} '.format(cpars.doCoag)
        g.write(line+'\n')

        line = '#pars::['+','.join(cparL)+']:: '
        for ckey in cparL:
            cval = getattr(cpars,ckey)
            if type(cval)==str:
                line += cval
            else:
                line += '{:10.3e}'.format(cval)
        g.write(line+'\n')

        Ncol, Nrow = data.shape
        sfmt = 2*'{:16.9e} '
        sfmt += (Nrow-2)*'{:12.5e} '
        for i in range(Npt):
            line = sfmt.format(*data[i,:])
            g.write(line+'\n')

        g.close()
    else:
        fg = pl.figure(figsize=(5,4))
        fg.subplots_adjust(top=0.9)
        ax1 = fg.add_subplot(111)
        ax1a = ax1.twiny()

        ax1.loglog(xc, parr, 'k-')
        ii = xc<0
        ax1.loglog(-xc[ii], parr[ii], 'k--')
        ax1.loglog(xv, parr, 'b-')
        ax1.loglog(xn, parr, '-', c='0.5')
        ax1.loglog(xeq, parr, 'b--')
        ax1.plot([cpars.xvbot], [parr[0]], 'bo', mec='b', mfc='b')
        ax1a.semilogy(Tarr, parr, 'r-')

        axs = [ax1,ax1a]
        for ax in axs:
            ax.set_ylim (parr[0]*1.01, parr[-1]/1.01)

        ax1.set_xlim(cpars.xvbot/1e8, 1e1*cpars.xvbot)

        ax1a.set_xlabel('T [K]', color='r')
        ax1.set_xlabel('concentration '+cpars.species)
        ax1.set_ylabel('pressure [cgs]')
        fg.savefig(cpars.name+cpars.species+'cloud.'+fmt)
        pl.close(fg)

        fg = pl.figure(figsize=(5,4.5))
        fg.subplots_adjust(top=0.88)
        axa = fg.add_subplot(111)
        axb = axa.twiny()
        l0 = axa.loglog( Mv, parr, ls='--', c='b')
        l1 = axa.loglog(-Mn, parr, ls='--', c='0.5')
        l4 = axa.loglog(-Mc, parr, ls='--', c='k')
        l3 = axb.loglog( xn, parr, c='0.5')
        l5 = axb.loglog( xc, parr, c='k')

        MX = max(-Mn.min(), -Mc.min(), Mv.max())

        axa.set_ylim(parr.max(), parr.min())
        axb.set_ylim(parr.max(), parr.min())
        axb.set_xlim(xn.max()/1e5, 1e5*xn.max())
        axa.set_xlim(MX/1e7, 2*MX)
        axa.set_ylabel('pressure')
        axa.set_xlabel('mass flux')
        axb.set_xlabel('concentration')

        axa.legend((l0[0],l1[0],l3[0],l4[0]),['Mv', 'Mn','xn','Mc'], loc=3)

        fg.savefig('test.'+fmt, dpi=180)
        pl.close(fg)

        fg = pl.figure(figsize=(5,4))
        fg.subplots_adjust(top=0.88)
        ax = fg.add_subplot(111)
        ax4b = ax.twiny()
        l0 = ax.loglog(ap*1e4, parr, 'k-')
        l1 = ax4b.loglog(tau, parr, '--', c='0.5')
        ax.set_ylim(parr.max(), parr.min())
        l2 = ax.axvline(1e4*cpars.aseed, ls='-', c='0.5', lw=1.5)
        ax.set_xlim(xmin=1e4*cpars.aseed/2.)
        ax.set_ylabel('pressure')
        ax.set_xlabel('grain radius [micron]')
        ax4b.set_xlabel('opt.depth')
        ax4b.set_xlim(tau.max()/1e5, 2*tau.max())

        ax.legend((l0[0],l1[0],l2),['ap', 'tau','a0'], loc=3)

        fg.savefig('test2.'+fmt, dpi=180)
        pl.close(fg)

    return data, shdrL



